import os
import logging
from pathlib import Path
from typing import TYPE_CHECKING, Any, Dict, Optional

from botocore.exceptions import ClientError
from botocore.client import Config
import boto3

from pydantic import BaseSettings, typing

if TYPE_CHECKING:
    from mypy_boto3_ssm.client import SSMClient


logger = logging.getLogger(__name__)


class AwsSsmSettingsSource:
    __slots__ = ("ssm_prefix",)

    def __init__(self, ssm_prefix: Optional[typing.StrPath]):
        self.ssm_prefix: Optional[typing.StrPath] = ssm_prefix

    @property
    def client(self) -> "SSMClient":
        return boto3.client("ssm", config=self.client_config)

    @property
    def client_config(self) -> Config:
        timeout = float(os.environ.get("SSM_TIMEOUT", 0.5))
        return Config(connect_timeout=timeout, read_timeout=timeout)

    def __call__(self, settings: BaseSettings) -> Dict[str, Any]:
        """
        Returns lazy SSM values for all settings.
        """
        secrets: Dict[str, Optional[Any]] = {}

        if self.ssm_prefix is None:
            return secrets

        secrets_path = Path(self.ssm_prefix)

        if not secrets_path.is_absolute():
            raise ValueError("SSM prefix must be absolute path")

        logger.debug(f"Building SSM settings with prefix of {secrets_path=}")

        try:
            paginator = self.client.get_paginator("get_parameters_by_path")
            response_iterator = paginator.paginate(
                Path=str(secrets_path), WithDecryption=True
            )

            output = {}
            for page in response_iterator:
                for parameter in page["Parameters"]:
                    key = Path(parameter["Name"]).relative_to(secrets_path).as_posix()
                    output[key] = parameter["Value"]
            return output

        except ClientError:
            logger.exception("Failed to get parameters from %s", secrets_path)
            return {}

    def __repr__(self) -> str:
        return f"AwsSsmSettingsSource(ssm_prefix={self.ssm_prefix!r})"



import logging
from typing import Tuple

from pydantic.env_settings import (
    EnvSettingsSource,
    InitSettingsSource,
    SecretsSettingsSource,
    SettingsSourceCallable,
)

from .source import AwsSsmSettingsSource

logger = logging.getLogger(__name__)


class AwsSsmSourceConfig:
    @classmethod
    def customise_sources(
        cls,
        init_settings: InitSettingsSource,
        env_settings: EnvSettingsSource,
        file_secret_settings: SecretsSettingsSource,
    ) -> Tuple[SettingsSourceCallable, ...]:

        return (
            init_settings,
            env_settings,
            # Usurping the `secrets_dir` arg. We can't expect any other args to
            # be passed to # the Settings module because Pydantic will complain
            # about unexpected arguments. `secrets_dir` comes from `_secrets_dir`,
            # one of the few special kwargs that Pydantic will allow:
            # https://github.com/samuelcolvin/pydantic/blob/45db4ad3aa558879824a91dd3b011d0449eb2977/pydantic/env_settings.py#L33
            AwsSsmSettingsSource(ssm_prefix=file_secret_settings.secrets_dir),
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydantic_ssm_settings']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.21.45,<2.0.0', 'pydantic>=1.6.2,<2.0.0']

setup_kwargs = {
    'name': 'pydantic-ssm-settings',
    'version': '0.0.1',
    'description': "Replace Pydantic's builtin Secret Support with a configuration provider that loads parameters from AWS Systems Manager Parameter Store.",
    'long_description': "# pydantic-ssm-settings\n\nReplace Pydantic's builtin [Secret Support](https://pydantic-docs.helpmanual.io/usage/settings/#secret-support) with a configuration provider that loads parameters from [AWS Systems Manager Parameter Store](https://docs.aws.amazon.com/systems-manager/latest/userguide/systems-manager-parameter-store.html). Parameters are loaded _lazily_, meaning that they are only requested from AWS if they are not provided via [standard field value priority](https://pydantic-docs.helpmanual.io/usage/settings/#field-value-priority) (i.e. initialiser, environment variable, or via `.env` file).\n\n## Usage\n\nThe simplest way to use this module is to inhert your settings `Config` class from `AwsSsmSourceConfig`. This will overwrite the [`file_secret_settings` settings source](https://pydantic-docs.helpmanual.io/usage/settings/#customise-settings-sources) with the `AwsSsmSettingsSource`. Provide a prefix to SSM parameters via the `_secrets_dir` initialiser value or the `secrets_dir` Config value.\n\n```py\nfrom pydantic import BaseSettings\nfrom pydantic_ssm_settings import AwsSsmSourceConfig\n\n\nclass WebserviceSettings(BaseSettings):\n    some_val: str\n    another_val: int\n\n    class Config(AwsSsmSourceConfig):\n        ...\n\nSimpleSettings(_secrets_dir='/prod/webservice')\n```\n\nThe above example will attempt to retreive values from `/prod/webservice/some_val` and `/prod/webservice/another_val` if not provided otherwise.",
    'author': 'Anthony Lukach',
    'author_email': 'anthonylukach@gmail.com',
    'maintainer': 'Anthony Lukach',
    'maintainer_email': 'anthonylukach@gmail.com',
    'url': 'https://github.com/developmentseed/pydantic-ssm-settings/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
